n=100; #the length of the permutation to generate
#Please uncomment the following line if you want to generate permutations of large length (greater than 100).
sys.setrecursionlimit(100000); #the default value is 1000.

#States of the S-T-automaton are labeled from 0 to card_states-1.
#O is the initial state, -1 is a void state using in the transition matrix below. 
#F is a vector with a 1 in position i iff i is a final state.
#Trans is the transition matrix of the automaton, it has two rows, one for transition labeled by S represented by 0 the other for transition labeled by T represented by 1. 
#Trans[l,q] is the state reached after reading l from q, it is -1 if the transition labeled by l is not available in q (e.g. Trans[0,1]=0 means that there is a transition labeled by S from 1 to 0).

#EX1 Language with Forbidden dd.
F=vector([1,0]);
Trans=matrix([[0,-1],[1,0]]);
card_states=2;
ST_enc=0;

#EX2 : Alternating permutation. (Uncomment the following lines)
#Trans=matrix([[-1],[0]]);
#F=vector([1]);
#card_states=1;
#ST_enc=1;#type of the S-T-encoding l=0 means l=a, l=1 means l=d 

#EX3 Identity permutation. (Simplex)
#Trans=matrix([[0],[-1]]);
#F=vector([1]);
#ST_enc=0;
#card_states=1;

#EX4 Language (aadd)^* up-up-down-down permutations.
#F=vector([0,1]);
#Trans=matrix([[-1,0],[1,-1]]);
#card_states=2;
#ST_enc=1;#type of the S-T-encoding l=0 means l=a, l=1 means l=d 

#EX5 
#F=vector([0,1,0]);
#Trans=matrix([[1,-1,-1],[-1,2,0]]);
#card_states=3;
#ST_enc=0;#type of the S-T-encoding l=0 means l=a, l=1 means l=d 

#EX5 
#F=vector([0,1,0]);
#Trans=matrix([[0,2,1],[1,-1,0]]);
#card_states=3;
#ST_enc=1;#type of the S-T-encoding l=0 means l=a, l=1 means l=d 


#M=block_matrix([[-M_s,-M_t],[M_t,M_s]]);
t_1=cputime();
#print('computing the polynomials');
#x=var('x');
R.<x>=RDF[];
qq=0;
v=matrix(RDF[x],card_states,n+1);
for q in [0..card_states-1]:
    v[q,0]=F[q];
for k in [1..n]:
    for q in [0..card_states-1]:
        qs=Trans[0,q] #qs is the successor of q by S
        if qs!=-1:
            pol=v[qs,k-1].integral();
            v[q,k]=v[q,k]+ pol(1)-pol(x);
        qt=Trans[1,q]; #qs is the successor of q by T
        if qt!=-1:
            pol=v[qt,k-1].integral();
            v[q,k]=v[q,k]+ pol(1-x)-pol(0);
#print transpose(v) #uncomment to see a matrix containing the polynomials v_{q,n}
print cputime(t_1);

t_2=cputime();
#print('generating the delays');
clock=vector(RDF,n+1);
location=vector(ZZ,n+1);
w=vector(ZZ,n+1);
m=vector(RDF,n+1);
delay=vector(RDF,n+1);
#t=var('t');
location[0]=0;
for k in [1..n]:
    f(x)=v[location[k-1],n-k+1]
    m[k]=f(clock[k-1])
    location[k]=0
    clock[k]=0.0
    qs=Trans[0,location[k-1]] #qs is the successor of q by S
    if qs!=-1:
        qt=Trans[1,location[k-1]]
        if qt==-1:
            b=1      #The only possible transition is S
            p=1
        else:
            pol(x)=v[qs,n-k].integral()
            p=(pol(1)-pol(clock[k-1]))/m[k]
            r=random()
            if r<p:
                b=1;
            else:
                b=0;
    else:
        b=0;      #The only possible transition is T
        qt=Trans[1,location[k-1]];
        p=0;
    if b==1:
        w[k]=0
        location[k]=qs
        r=random()
        pol=v[qs,n-k].integral()
        pol2(x)=(pol(clock[k-1]+x)-pol(clock[k-1]))/(m[k]*p) -r
        delay[k]=find_root(pol2(x),0,1-clock[k-1])
        clock[k]=clock[k-1]+delay[k]
    else:
        w[k]=1
        location[k]=qt
        r=random()
        pol=v[qt,n-k].integral()
        pol2(x)=(pol(x)-pol(0))/(m[k]*(1-p)) -r
        delay[k]=find_root(pol2(x),0,1-clock[k-1])
        clock[k]=delay[k]
#print delay  #uncomment to see the vector of delays

#print cputime(t_2);
t_3=cputime();

#print('computing the function phi');
#computing the function phi
w[0]=ST_enc; #type of the S-T-encoding l=0 means l=a, l=1 means l=d 

nu=vector(RDF,n);
if n!=0:
    curad=ST_enc*(1-w[0])+(1-ST_enc)*w[0];
    nu[0]=curad*(1-delay[0])+(1-curad)*delay[0]; #type of the S-T-encoding l=0 means l=a, l=1 means l=d 
    for k in range(1,n):#a changer si n=0
        if w[k]==0:                                       #a straight 
            if curad==0:                                  #The previous letter is an ascent: pattern aa
                nu[k]=nu[k-1]+delay[k]
            else:                                           #The previous letter is a descent: pattern dd
                nu[k]=nu[k-1]-delay[k]
        else:        
            if curad==0:                                     #The previous letter is an ascent: pattern ad
                nu[k]=1-delay[k]
            else:                                           #The previous letter is a descent: pattern da
                nu[k]=delay[k]
            curad=1-curad;                                  #flip the value of l since w_k is a turn
#print cputime(t_3);

t_4=cputime();
#print('sorting and answering');
answer=Word(list(nu)[0:n]).standard_permutation();
print answer;
#answer=Word(list(nu)[1:n+1]).standard_permutation();print answer #return the permutation
#print cputime(t_4);
#print cputime(t_1);

